<?php
/**
 * Forwards_Driver_ldapadmin:: implements the Forwards_Driver API for LDAP
 * driven mail servers using a standard LDAP configuration with Admin
 * user enabled as opposed to allowing users access to the database.
 *
 * This driver is merely a heavily modified version of the standard LDAP
 * driver included with Forwards.  I've left the original comment block
 * from that driver.
 *
 * TODO List:
 *	- Add ability to accept the users password for authentication
 *	  before changing their deliverMailbox
 *
 * author	Owen Wagoner
 *
 **** Old ldap.php comments below ****
 *
 * 
 * Forwards_Driver_ldap:: implements the Forwards_Driver API for LDAP driven
 * mail servers.
 *
 * $Horde: forwards/lib/Driver/ldap.php,v 1.10 2006/02/16 16:34:43 jan Exp $
 *
 * Copyright 2001-2006 Eric Rostetter <eric.rostetter@physics.utexas.edu>
 *
 * See the enclosed file LICENSE for license information (ASL). If you
 * did not receive this file, see http://www.horde.org/licenses/asl.php.
 *
 * @author  Eric Rostetter <eric.rostetter@physics.utexas.edu>
 * @package Forwards
 */
class Forwards_Driver_ldapadmin extends Forwards_Driver {

    /**
     * Pointer to the ldap connection.
     *
     * @var resource
     */
    var $_ds;

    /**
     * Check if the realm has a specific configuration.
     *
     * If not, try to fall back on the default configuration.  If
     * still not a valid configuration then exit with an error.
     *
     * @param string $realm  The realm of the user, or "default" if none.
     *                       Note: passed by reference so we can change
     *                       its value!
     */
    function checkConfiguration(&$realm)
    {
        // If no realm passed in, or no host config for the realm
        // passed in, then we fall back to the default realm.
        if (empty($realm) || empty($this->_params[$realm]['server'])) {
            $realm = 'default';
        }

        // If still no host/port, then we have a misconfigured module.
        if (empty($this->_params[$realm]['host']) ||
            empty($this->_params[$realm]['port']) ) {
            $this->_error = _("The module is not properly configured!");
            return false;
        }
        return true;
    }

    /**
     * Begins forwarding of mail for a user.
     *
     * @param string $user        The username of the user.
     * @param string $realm       The realm of the user.
     * @param string $password    The password of the user.
     * @param string $target      The email address that mail should be
     *                            forwarded to.
     * @param boolean $keeplocal  Keep a copy of forwarded mail in the local
     *                            mailbox. (Ignored)
     *
     * @return boolean  True on success, false otherwise.
     */
    function enableForwarding($user, $realm, $password, $target, $keeplocal)
    {
        // Make sure the configuration file is correct.
        if (!$this->checkConfiguration($realm)) {
            return false;
        }

	// Find out what the admin settings are
	$lrealm = $this->_params[$realm]['realm'];
	$admin = $this->_params[$realm]['admindn'];
	$adminpw = $this->_params[$realm]['adminpw'];
	$domains = $this->_params[$realm]['domains'];
	
	/*
	 * Check the target domain to verify that it is
	 * not disallowed by the configuration.  This can be
	 * used to prevent users from forwarding their mail
	 * to someone else in the same domain as well as
	 * preventing users from causing mail loops by
	 * forwarding mail to themselves.
	 *
	 * Use str_replace to remove spaces (spaces aren't
	 * allowed in domains so they *should* have only been
	 * input by the user for readability.
	 *
	 * Use explode to take the new string from str_replace
	 * and split it up by comma for the domain array
	 */
	$domain = explode(",", str_replace(" ", "", $domains));
	// Check to verify that target email address is in
	// the correct format.
	/***
	  This is here becuase I didn't see it anywhere
	  in the regular forwards code.  If it is, this
	  should be removed
	***/
	$pattern = '/^[a-z0-9._-]+@[a-z0-9.-]+\.[a-z]{2,4}$/i';
	if(!preg_match($pattern, $target)) {
		// Email address isn't valid so raise an error
		$this->_error = "Invalid or unknown email address format";
		return false;
	}
	// Pull the domain out of the target
	// Some error checking is left out because of the email
	// validation already performed
	$tdomainx = substr(strstr($target, '@'), 1);

	/*
	 * Check the format of the domain section
	 *
	 * This is done to prevent the admin from needing to
	 * enter all the possible host names associated with
	 * a restricted domain.  All they need to enter is the
	 * domain name ("domain.net" for example) and then all
	 * hosts in the domain are blocked.  IP addresses are
	 * not considered so there is still the possiblity of
	 * a really smart user doing something malicious but
	 * there was nothing in the old driver to prevent that
	 * either.
	 *
	 */
	$dpattern = '/[a-z0-9-]+\.[a-z0-9.-]+\.[a-z]{2,4}$/i';
	if(!preg_match($dpattern, $tdomainx)) {
		// The domain is in the format "domain.com"
		$tdomain = $tdomainx;
	} else {
		// The domain is in the format "host.domain.com"
		// So we retrieve only the last 2 parts "domain.com"
		$darray = explode(".", $tdomainx);
		$num = count($darray);
		$tdomain = $darray[$num - 2] . "." . $darray[$num -1];
	}

	// Compare target domain to unavailable domains
	foreach ( $domain as $domainx ) {
		if ( $tdomain == $domainx ) {
			// This domain is one of the restricted ones
			$this->_error = "Forwarding to a restricted ";
			$this->_error .= "domain is prohibited.";
			return false;
		}
	}
        
        // Connect as the admin user.
        $res = $this->_connect($admin, $adminpw, $realm);
        if (is_a($res, 'PEAR_Error')) {
            $this->_error = $res->getMessage();
            $this->_error .= ' - ' .  _("Check your password");
            return false;
        }
        
	// Change the user's forwards.
	$newDetails[$this->_params[$realm]['forwards']] = $target;
	$userid = $this->_params[$realm]['uid'];
        $res = ldap_modify($this->_ds, "$userid=$user,$lrealm" , $newDetails);
        if (!$res) {
            $res = PEAR::raiseError(ldap_error($this->_ds));
        }

        // Disconnect from the ldap server.
        $this->_disconnect();

        return true;
    }

    /**
     * Stops forwarding of mail for a user.
     *
     * @param string $user      The username of the user.
     * @param string $realm     The realm of the user.
     * @param string $password  The password of the user.
     *
     * @return boolean  True on success, false otherwise.
     */
    function disableForwarding($user, $realm, $password)
    {
        // Make sure the configuration file is correct.
        if (!$this->checkConfiguration($realm)) {
            return false;
        }

	// Find out what the admin settings are
        $lrealm = $this->_params[$realm]['realm'];
        $admin = $this->_params[$realm]['admindn'];
        $adminpw = $this->_params[$realm]['adminpw'];
	$userid = $this->_params[$realm]['uid'];

        // Connect as the admin user.
        $res = $this->_connect($admin, $adminpw, $realm);
        if (is_a($res, 'PEAR_Error')) {
            $this->_error = $res->getMessage();
            $this->_error .= ' - ' .  _("Check your password");
            return false;
        }
        
	// Reset the deliverMailbox to be the user id
        $newDetails[$this->_params[$realm]['forwards']] = $user;
        $res = ldap_modify($this->_ds, "$userid=$user,$lrealm", $newDetails);
        if (!$res) {
            $res = PEAR::raiseError(ldap_error($this->_ds));
        }

        // Disconnect from the ldap server.
        $this->_disconnect();

        return $res;
    }

    /**
     * Retrieves current state of mail redirection for a user.
     *
     * @param string $user      The username of the user.
     * @param string $realm     The realm of the user.
     * @param string $password  The password of the user.
     *
     * @return mixed  'Y' if forwarding is enabled, or false otherwise.
     */
    function isEnabledForwarding($user, $realm, $password)
    {
        // Make sure the configuration file is correct.
        if (!$this->checkConfiguration($realm)) {
            return false;
        }

	// Find out what the admin settings are
        $lrealm = $this->_params[$realm]['realm'];
        $admin = $this->_params[$realm]['admindn'];
        $adminpw = $this->_params[$realm]['adminpw'];
        $userid = $this->_params[$realm]['uid'];

        // Connect as the admin user.
        $res = $this->_connect($admin, $adminpw, $realm);
        if (is_a($res, 'PEAR_Error')) {
            $this->_disconnect();
            if ($res->getMessage() == _("Could not bind to ldap server")) {
                return PEAR::raiseError(_("Incorrect Password"));
            }
            return $res;
        }

	/*
	 * Note:  This probably won't be as organizaion independent
	 * as I would like it to be but this is the best I could come
	 * up with and still have it work with our organization.  If
	 * your organization has different requirements then you should
	 * modify this section of the driver
	 */

	// Get the LDAP forward mailbox parameter
        $attribs = array($this->_params[$realm]['forwards']);
	// Get the value for forwards 
	$delivermbx = $this->_getForwards($user, $attribs);
	// Disconnect
	$this->_disconnect();
	
	// Decide whether or not forwarding is turned on
	if ( $delivermbx == $user ) {
		// Forwarding is not turned on
        	return 'N';
	} else {
		// Forwarding is enabled
		return 'Y';
	}
    }

    /**
     * Retrieves current target of mail redirection for a user.
     *
     * @param string $user      The username of the user.
     * @param string $realm     The realm of the user.
     * @param string $password  The password of the user.
     *
     * @return string  The current forwarding mail address, or false on error.
     */
    function currentTarget($user, $realm = 'default', $password)
    {
        // Make sure the configuration file is correct.
        if (!$this->checkConfiguration($realm)) {
            return false;
        }

	// Find out what the admin settings are
        $lrealm = $this->_params[$realm]['realm'];
        $admin = $this->_params[$realm]['admindn'];
        $adminpw = $this->_params[$realm]['adminpw'];

        // Connect as the admin user.
        $res = $this->_connect($admin, $adminpw, $realm);
        if (is_a($res, 'PEAR_Error')) {
            $this->_disconnect();
            if ($res->getMessage() == _("Could not bind to ldap server")) {
                return PEAR::raiseError(_("Incorrect Password"));
            }
            return $res;
        }

        $attribs = array($this->_params[$realm]['forwards']);

        return $this->_getForwards($user, $attribs);
    }

    /**
     * Lookup and return the user's dn.
     *
     * @param string $user   The username of the user.
     * @param string $realm  The realm of the user.
     *
     * @return string  The ldap dn for the user.
     */
    function _lookupdn($user, $realm)
    {
	// Find out what some of the settings are
        $lrealm = $this->_params[$realm]['realm'];
        $userid = $this->_params[$realm]['uid'];

        return "$userid=$user,$lrealm";
    }

    function _getForwards($user, $attribs, $realm = 'default')
    {
	// Basedn
	$basedn = $this->_params[$realm]['basedn'];

        $sr = ldap_search($this->_ds, $basedn, "uid=$user", $attribs);
        $entry = ldap_first_entry($this->_ds, $sr);
        $res = ldap_get_attributes($this->_ds, $entry);
        if ($res['count'] == 0) {
            return false;
        }

        $values = ldap_get_values($this->_ds, $entry, $attribs[0]);
        return $values[0];
    }

    /**
     * Connects to an LDAP server and binds as the guest user or as the
     * optional userdn.
     *
     * @param string $userdn    The DN to use when binding non-anonymously.
     * @param string $password  The password for $userdn.
     * @param string $realm     The realm of the user.
     *
     * @return boolean  True on success, false otherwise.
     */
    function _connect($userdn = null, $password = null, $realm = 'default')
    {
        $this->_ds = ldap_connect($this->_params[$realm]['host'], $this->_params[$realm]['port']);
        if (!$this->_ds) {
            return PEAR::raiseError(_("Could not connect to ldap server"));
        }
        if (isset($this->_params[$realm]['version'])) {
            ldap_set_option($this->_ds, LDAP_OPT_PROTOCOL_VERSION,
                            $this->_params[$realm]['version']);
        }

        if (!is_null($userdn)) {
            $result = @ldap_bind($this->_ds, $userdn, $password);
        } else {
	    $userdn = $this->_params[$realm]['admindn'];
	    $password = $this->_params[$realm]['adminpw'];
            $result = @ldap_bind($this->_ds, $userdn, $password);
        }

        if (!$result) {
            return PEAR::raiseError(_("Could not bind to ldap server"));
        }

        return true;
    }

    /**
     * Close the ldap connection.
     */
    function _disconnect()
    {
        @ldap_close($this->_ds);
    }

}
